@@ -16,8 +16,8 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 59754 2009-03-23 01:01:11Z CJP $"
-__revision__ = "$Revision: 59754 $"
+__svnid__ = "$Id: Editra.py 60129 2009-04-14 02:02:17Z CJP $"
+__revision__ = "$Revision: 60129 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -756,6 +756,8 @@ def InitConfig():
 
     # Resolve resource locations
     ed_glob.CONFIG['CONFIG_DIR'] = util.ResolvConfigDir(u"")
+    ed_glob.CONFIG['INSTALL_DIR'] = util.ResolvConfigDir(u"", True)
+    ed_glob.CONFIG['KEYPROF_DIR'] = util.ResolvConfigDir(u"ekeys", True)
     ed_glob.CONFIG['SYSPIX_DIR'] = util.ResolvConfigDir(u"pixmaps", True)
     ed_glob.CONFIG['PLUGIN_DIR'] = util.ResolvConfigDir(u"plugins")
     ed_glob.CONFIG['THEME_DIR'] = util.ResolvConfigDir(os.path.join(u"pixmaps", u"theme"))
@@ -961,7 +963,11 @@ def _Main(opts, args):
     # But not if there are command line args for files to open
     if profiler.Profile_Get('SAVE_SESSION', 'bool', False) and not len(args):
         session = profiler.Profile_Get('LAST_SESSION', default=u'')
-        frame.GetNotebook().LoadSessionFile(session)
+        if isinstance(session, list):
+            # Check for format conversion from previous versions
+            profiler.Profile_Set('LAST_SESSION', u'')
+        else:
+            frame.GetNotebook().LoadSessionFile(session)
 
     # Unlike wxMac/wxGTK Windows doesn't post an activate event when a window
     # is first shown, so do it manually to make sure all event handlers get
@@ -971,7 +977,8 @@ def _Main(opts, args):
 
     # Do update check, only check if its been more than a day since the last
     # check
-    if profiler.Profile_Get('CHECKUPDATE', default=True):
+    isadmin = os.access(ed_glob.CONFIG['INSTALL_DIR'], os.R_OK|os.W_OK)
+    if isadmin and profiler.Profile_Get('CHECKUPDATE', default=True):
         uthread = updater.UpdateThread(editra_app, ID_UPDATE_CHECK)
         uthread.start()
 
