@@ -12,8 +12,8 @@ Simple autocompletion support for HTML and XML documents.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: htmlcomp.py 59583 2009-03-17 03:49:44Z CJP $"
-__revision__ = "$Revision: 59583 $"
+__cvsid__ = "$Id: htmlcomp.py 60378 2009-04-26 04:55:23Z CJP $"
+__revision__ = "$Revision: 60378 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -121,9 +121,10 @@ class Completer(completer.BaseCompleter):
         # Check if we are completing an open tag
         if tmp.endswith('<'):
             if buff.GetLexer() == wx.stc.STC_LEX_XML:
-                return _FindXmlTags(buff.GetText())
+                taglst = _FindXmlTags(buff.GetText())
             else:
-                return TAGS
+                taglst = TAGS
+            return WrapTagImages(taglst)
 
         tmp = tmp.rstrip('>').rstrip()
         if len(tmp) and (tmp[-1] in '"\' \t' or tmp[-1].isalpha()):
@@ -168,3 +169,11 @@ def _FindXmlTags(text):
     else:
         matches = [u'!--', ]
     return matches
+
+def WrapTagImages(taglst):
+    """Apply image identifiers to completion strings
+    @return: list
+
+    """
+    tagstr = u"?%d" % completer.IMG_ELEMENT
+    return [ tag + tagstr for tag in taglst ]
