@@ -16,8 +16,8 @@ deduct the requested information.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: pycomp.py 59583 2009-03-17 03:49:44Z CJP $"
-__revision__ = "$Revision: 59583 $"
+__cvsid__ = "$Id: pycomp.py 60376 2009-04-26 04:35:42Z CJP $"
+__revision__ = "$Revision: 60376 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -96,7 +96,23 @@ class Completer(completer.BaseCompleter):
             else:
                 # Get Autocompletion List
                 complst = cmpl.get_completions(command)
-                sigs = [ sig['word'].rstrip('(.') for sig in complst ]
+                sigs = list()
+                for sig in complst:
+                    word = sig['word'].rstrip(u'(.')
+                    if sig['type'] == "function":
+                        word += u'?%d' % completer.IMG_FUNCTION
+                    elif sig['type'] == "method":
+                        word += u'?%d' % completer.IMG_METHOD
+                    elif sig['type'] == "class":
+                        word += u'?%d' % completer.IMG_CLASS
+                    elif sig['type'] == "attribute":
+                        word += u'?%d' % completer.IMG_ATTRIBUTE
+                    elif sig['type'] == "property":
+                        word += u'?%d' % completer.IMG_PROPERTY
+
+                    sigs.append(word)
+
+#                sigs = [ sig['word'].rstrip('(.') for sig in complst ]
                 sigs.sort(lambda x, y: cmp(x.upper(), y.upper()))
                 return sigs
 
@@ -258,6 +274,7 @@ class PyCompleter(object):
 
             dbg("[pycomp] completing: stmt:%s" % stmt)
             completions = []
+            isdict = isinstance(compdict, dict)
             for meth in compdict:
                 if meth == "_PyCmplNoType":
                     continue #this is internal
@@ -281,22 +298,38 @@ class PyCompleter(object):
                         else:
                             doc = ' '
 
+                        if isdict:
+                            typestr = str(compdict[inst])
+                        else:
+                            typestr = str(inst)
+
                         comp = {'word' : meth,
                                 'abbr' : meth,
-                                'info' : _cleanstr(str(doc))}
+                                'info' : _cleanstr(str(doc)),
+                                'type' : typestr}
 
-                        typestr = str(inst)
                         if "function" in typestr:
                             comp['word'] += '('
                             comp['abbr'] += '(' + _cleanstr(self.get_arguments(inst))
+                            comp['type'] = "function"
                         elif "method" in typestr:
                             comp['word'] += '('
                             comp['abbr'] += '(' + _cleanstr(self.get_arguments(inst))
+                            comp['type'] = "method"
                         elif "module" in typestr:
                             comp['word'] += '.'
+                            comp['type'] = "module"
                         elif "class" in typestr:
                             comp['word'] += '('
                             comp['abbr'] += '('
+                            comp['type'] = "class"
+                        elif "attribute" in typestr:
+                            comp['type'] = "attribute"
+                        elif "property" in typestr:
+                            comp['type'] = "property"
+#                        else:
+#                            print typestr, meth
+
                         completions.append(comp)
                 except Exception, msg:
                     dbg("[pycomp][err] inner completion: %s [stmt='%s']:" % (msg, stmt))
@@ -835,7 +868,10 @@ class PyParser:
                         stmt = self._parseassignment()
                         dbg("[pycomp] parseassignment: %s = %s" % (name, stmt))
                         if stmt != None:
-                            self.scope.local("%s = %s" % (name, stmt))
+                            # XXX Safety Check don't allow assigments to 
+                            # item attributes unless the attribute is self
+                            if u'.' not in name or name.startswith(u'self'):
+                                self.scope.local("%s = %s" % (name, stmt))
                     freshscope = False
         except StopIteration: #thrown on EOF
             pass
