@@ -33,13 +33,17 @@ PB_STYLE_SQUARE:
 Instead of the default rounded shape use a rectangular shaped button with
 square edges.
 
-PB_STYLE_NB:
+PB_STYLE_NOBG:
 This style only has an effect on Windows but does not cause harm to use on the
 platforms. It should only be used when the control is shown on a panel or other
 window that has a non solid color for a background. i.e a gradient or image is
 painted on the background of the parent window. If used on a background with
 a solid color it may cause the control to loose its transparent appearance.
 
+PB_STYLE_DROPARROW:
+Add a drop button arrow to the button that will send a separate event when
+clicked on.
+
 Other attributes can be configured after the control has been created. The
 settings that are currently available are as follows:
 
@@ -69,21 +73,22 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: platebtn.py 57345 2008-12-14 23:47:52Z CJP $"
-__revision__ = "$Revision: 57345 $"
+__svnid__ = "$Id: platebtn.py 59613 2009-03-18 20:52:20Z CJP $"
+__revision__ = "$Revision: 59613 $"
 
-__all__ = ["PlateButton", "AdjustAlpha", "AdjustColor", "GetHighlightColor",
+__all__ = ["PlateButton",
            "PLATE_NORMAL", "PLATE_PRESSED", "PLATE_HIGHLIGHT", 
            "PB_STYLE_DEFAULT", "PB_STYLE_GRADIENT", "PB_STYLE_SQUARE",
-           "PB_STYLE_NOBG" ]
+           "PB_STYLE_NOBG", "PB_STYLE_DROPARROW",
+           "EVT_PLATEBTN_DROPARROW_PRESSED"]
 
 #-----------------------------------------------------------------------------#
 # Imports
 import wx
+import wx.lib.newevent
 
-# Used on OSX to get access to carbon api constants
-if wx.Platform == '__WXMAC__':
-    import Carbon.Appearance
+# Local Imports
+from eclutil import *
 
 #-----------------------------------------------------------------------------#
 # Button States
@@ -97,65 +102,11 @@ PB_STYLE_GRADIENT = 2   # Gradient Filled Background
 PB_STYLE_SQUARE   = 4   # Use square corners instead of rounded
 PB_STYLE_NOBG     = 8   # Usefull on Windows to get a transparent appearance
                         # when the control is shown on a non solid background
+PB_STYLE_DROPARROW = 16 # Draw drop arrow and fire EVT_PLATEBTN_DROPRROW_PRESSED event
 
 #-----------------------------------------------------------------------------#
-# Utility Functions
-def AdjustAlpha(colour, alpha):
-    """Adjust the alpha of a given colour"""
-    return wx.Colour(colour.Red(), colour.Green(), colour.Blue(), alpha)
 
-def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
-    """ Brighten/Darken input colour by percent and adjust alpha
-    channel if needed. Returns the modified color.
-    @param color: color object to adjust
-    @type color: wx.Color
-    @param percent: percent to adjust +(brighten) or -(darken)
-    @type percent: int
-    @keyword alpha: amount to adjust alpha channel
-
-    """
-    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
-                         for val in color.Get() ]
-
-    if percent < 0:
-        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
-    else:
-        radj, gadj, badj = [ val or 255 for val in [radj, gadj, badj] ]
-
-    red = min(color.Red() + radj, 255)
-    green = min(color.Green() + gadj, 255)
-    blue = min(color.Blue() + badj, 255)
-    return wx.Colour(red, green, blue, alpha)
-
-def BestLabelColour(color):
-    """Get the best color to use for the label that will be drawn on
-    top of the given color.
-    @param color: background color that text will be drawn on
-
-    """
-    avg = sum(color.Get()) / 3
-    if avg > 192:
-        txt_color = wx.BLACK
-    elif avg > 128:
-        txt_color = AdjustColour(color, -95)
-    elif avg < 64:
-        txt_color = wx.WHITE
-    else:
-        txt_color = AdjustColour(color, 95)
-    return txt_color
-
-def GetHighlightColour():
-    """Get the default highlight color
-    @return: wx.Color
-
-    """
-    if wx.Platform == '__WXMAC__':
-        brush = wx.Brush(wx.BLACK)
-        # kThemeBrushButtonPressedLightHighlight
-        brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
-        return brush.GetColour()
-    else:
-        return wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
+PlateBtnDropArrowPressed, EVT_PLATEBTN_DROPARROW_PRESSED = wx.lib.newevent.NewEvent()
 
 #-----------------------------------------------------------------------------#
 
@@ -240,7 +191,7 @@ class PlateButton(wx.PyControl):
         @param ypos: y cord to start at
 
         """
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             # Positioning needs a little help on Windows
             if wx.Platform == '__WXMSW__':
                 xpos -= 2
@@ -325,19 +276,17 @@ class PlateButton(wx.PyControl):
             self.__DrawHighlight(gc, width, height)
 
         elif self._state['cur'] == PLATE_PRESSED:
-            gc.SetTextForeground(self._color['ptxt'])
+            gc.SetTextForeground(self._color['htxt'])
             if wx.Platform == '__WXMAC__':
-                brush = wx.Brush(wx.BLACK)
-                brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
-                pen = wx.Pen(brush.GetColour(), 1, wx.SOLID)
+                pen = wx.Pen(GetHighlightColour(), 1, wx.SOLID)
             else:
                 pen = wx.Pen(AdjustColour(self._color['press'], -80, 220), 1)
             gc.SetPen(pen)
 
+            self.__DrawHighlight(gc, width, height)
             txt_x = self.__DrawBitmap(gc)
             gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
             self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
-            self.__DrawHighlight(gc, width, height)
 
         else:
             if self.IsEnabled():
@@ -355,12 +304,11 @@ class PlateButton(wx.PyControl):
     def __InitColors(self):
         """Initialize the default colors"""
         color = GetHighlightColour()
-        pcolor = AdjustColour(color, -10, 160)
+        pcolor = AdjustColour(color, -12)
         colors = dict(default=True,
                       hlight=color, 
                       press=pcolor,
-                      htxt=BestLabelColour(self.GetForegroundColour()),
-                      ptxt=self.GetForegroundColour())
+                      htxt=BestLabelColour(self.GetForegroundColour()))
         return colors
 
     #---- End Private Member Function ----#
@@ -412,7 +360,7 @@ class PlateButton(wx.PyControl):
         else:
             width += 10
 
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             width += 12
 
         best = wx.Size(width, height)
@@ -528,10 +476,16 @@ class PlateButton(wx.PyControl):
         pos = evt.GetPositionTuple()
         self.SetState(PLATE_PRESSED)
         size = self.GetSizeTuple()
-        if self._menu is not None and pos[0] >= size[0] - 16:
-            self.ShowMenu()
+        if pos[0] >= size[0] - 16:
+            if self._menu is not None:
+                self.ShowMenu()
+            elif self._style & PB_STYLE_DROPARROW:
+                event = PlateBtnDropArrowPressed()
+                event.SetEventObject(self)
+                wx.PostEvent(self, event)
+        
         self.SetFocus()
-            
+
     def OnLeftUp(self, evt):
         """Post a button event if the control was previously in a
         pressed state.
@@ -539,7 +493,10 @@ class PlateButton(wx.PyControl):
 
         """
         if self._state['cur'] == PLATE_PRESSED:
-            self.__PostEvent()
+            pos = evt.GetPositionTuple()
+            size = self.GetSizeTuple()
+            if not (self._style & PB_STYLE_DROPARROW and pos[0] >= size[0] - 16):
+                self.__PostEvent()
         self.SetState(PLATE_HIGHLIGHT)
 
     def OnMenuClose(self, evt):
@@ -600,7 +557,7 @@ class PlateButton(wx.PyControl):
         wx.PyControl.SetLabel(self, label)
         self.InvalidateBestSize()
 
-    def SetLabelColor(self, normal, hlight=wx.NullColor, press=wx.NullColor):
+    def SetLabelColor(self, normal, hlight=wx.NullColor):
         """Set the color of the label. The optimal label color is usually
         automatically selected depending on the button color. In some
         cases the colors that are choosen may not be optimal.
@@ -612,7 +569,6 @@ class PlateButton(wx.PyControl):
 
         @param normal: Label color for normal state
         @keyword hlight: Color for when mouse is hovering over
-        @keyword press: Color for when button is pressed
 
         """
         self._color['default'] = False
@@ -624,13 +580,10 @@ class PlateButton(wx.PyControl):
             else:
                 self._color['htxt'] = BestLabelColour(normal)
 
-        if press is not None:
-            if press.IsOk():
-                self._color['ptxt'] = press
-            else:
-                self._color['ptxt'] = BestLabelColour(self._color['press'])
-
-        self.GetParent().RefreshRect(self.GetRect(), False)
+        if wx.Platform == '__WXMSW__':
+            self.GetParent().RefreshRect(self.GetRect(), False)
+        else:
+            self.Refresh()
 
     def SetMenu(self, menu):
         """Set the menu that can be shown when clicking on the
@@ -659,7 +612,6 @@ class PlateButton(wx.PyControl):
             self._color['hlight'] = color
         self._color['press'] = AdjustColour(color, -10, 160)
         self._color['htxt'] = BestLabelColour(self._color['hlight'])
-        self._color['ptxt'] = BestLabelColour(self._color['htxt'])
         self.Refresh()
 
     def SetState(self, state):
@@ -670,7 +622,10 @@ class PlateButton(wx.PyControl):
         """
         self._state['pre'] = self._state['cur']
         self._state['cur'] = state
-        self.GetParent().RefreshRect(self.GetRect(), False)
+        if wx.Platform == '__WXMSW__':
+            self.GetParent().RefreshRect(self.GetRect(), False)
+        else:
+            self.Refresh()
 
     def SetWindowStyle(self, style):
         """Sets the window style bytes, the updates take place
