@@ -15,8 +15,8 @@ syntax highlighting of all supported filetypes.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_basestc.py 59714 2009-03-22 01:31:07Z CJP $"
-__revision__ = "$Revision: 59714 $"
+__svnid__ = "$Id: ed_basestc.py 60376 2009-04-26 04:35:42Z CJP $"
+__revision__ = "$Revision: 60376 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -32,6 +32,7 @@ import ed_txt
 from syntax import syntax
 from autocomp import autocomp
 from extern import vertedit
+from profiler import Profile_Get
 
 #-----------------------------------------------------------------------------#
 
@@ -90,6 +91,10 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             for keys in _GetMacKeyBindings():
                 self.CmdKeyAssign(*keys)
 
+        # Setup Autocomp images
+        # TODO: should be called on theme change messages
+        self.RegisterImages()
+
         # Event Handlers
         self.Bind(wx.stc.EVT_STC_CHANGE, self.OnChanged)
         self.Bind(wx.stc.EVT_STC_MODIFIED, self.OnModified)
@@ -213,8 +218,8 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """
         self.AutoCompSetAutoHide(False)
         self.AutoCompSetChooseSingle(True)
-#        self._code['compsvc'].LoadCompProvider(self.GetLexer())
-        self._code['compsvc'] = autocomp.AutoCompService.GetCompleter(self)
+        extend = Profile_Get('AUTO_COMP_EX') # Using extended autocomp?
+        self._code['compsvc'] = autocomp.AutoCompService.GetCompleter(self, extend)
         self.AutoCompSetIgnoreCase(not self._code['compsvc'].GetCaseSensitive())
         self.AutoCompStops(self._code['compsvc'].GetAutoCompStops())
         # TODO: come back to this it can cause some annoying behavior where
@@ -401,9 +406,10 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         @todo: fillups are currently disabled. See note in Configure.
 
         """
-        curr_pos = self.GetCurrentPos()
-        start = curr_pos - 1
-        col = self.GetColumn(curr_pos)
+        # NOTE: the column position retuned by GetCurLine is not correct
+        #       for multibyte characters.
+        line, col = self.GetCurLine()
+        col = self.GetColumn(self.GetCurrentPos())
         cmd_lmt = list(self._code['compsvc'].GetAutoCompStops() + \
                        self._code['compsvc'].GetAutoCompFillups())
         for key in self._code['compsvc'].GetAutoCompKeys():
@@ -411,16 +417,14 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             if kval in cmd_lmt:
                 cmd_lmt.remove(kval)
 
-        while col - (curr_pos - start) > 0:
-            txt = self.GetTextRange(start, curr_pos)
-            if len(txt) and txt[0] not in cmd_lmt:
-                start -= 1
+        curr_pos = col - 1
+        cmd = u''
+        while curr_pos > -1:
+            cmd = line[curr_pos:col]
+            if len(cmd) and cmd[0] not in cmd_lmt:
+                curr_pos -= 1
             else:
                 break
-
-        if self.GetColumn(start) != 0:
-            start += 1
-        cmd = self.GetTextRange(start, curr_pos)
         return cmd.strip()
 
     def GetCommentChars(self):
@@ -594,6 +598,19 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         else:
             evt.Skip()
 
+    def RegisterImages(self):
+        """Register the images for the autocomp popup list"""
+        images = [(autocomp.IMG_FUNCTION, ed_glob.ID_FUNCT_TYPE),
+                  (autocomp.IMG_METHOD, ed_glob.ID_METHOD_TYPE),
+                  (autocomp.IMG_PROPERTY, ed_glob.ID_PROPERTY_TYPE),
+                  (autocomp.IMG_CLASS, ed_glob.ID_CLASS_TYPE),
+                  (autocomp.IMG_VARIABLE, ed_glob.ID_VARIABLE_TYPE),
+                  (autocomp.IMG_ELEMENT, ed_glob.ID_ELEM_TYPE)]
+        for idx, img in images:
+            bmp = wx.ArtProvider.GetBitmap(str(img), wx.ART_MENU)
+            if bmp.IsOk():
+                self.RegisterImage(idx, bmp)
+
     def SetDocument(self, doc):
         """Change the document object used.
         @param doc: an L{ed_txt.EdFile} instance
@@ -703,8 +720,11 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         lst = self._code['compsvc'].GetAutoCompList(command)
         if lst is not None and len(lst):
             self.BeginUndoAction()
-            self.AutoCompShow(pos - self.WordStartPosition(pos, True),
-                              u' '.join(lst))
+            lst = u' '.join(lst)
+            if lst.isspace():
+                return
+
+            self.AutoCompShow(pos - self.WordStartPosition(pos, True), lst)
 
             if len(lst) == 1 and self._code['compsvc'].GetAutoCompAfter():
                 super(EditraBaseStc, self).GotoPos(pos)
@@ -807,8 +827,10 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 def _GetMacKeyBindings():
     """Returns a list of 3-element tuples defining the standard key
     bindings for Mac text editors -- i.e., the behavior of option-arrow,
-    shift-delete, and so on.  Each tuple consists of:
-        (key code, modifier keys, STC action)
+    shift-delete, and so on.
+
+    @return: list of (key code, modifier keys, STC action)
+
     """
     # A good reference for these: http://www.yellowbrain.com/stc/keymap.html
     return [
