@@ -15,8 +15,8 @@ main Ui component of the editor that contains all the other components.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 59569 2009-03-16 03:40:34Z CJP $"
-__revision__ = "$Revision: 59569 $"
+__svnid__ = "$Id: ed_main.py 60367 2009-04-26 00:20:46Z CJP $"
+__revision__ = "$Revision: 60367 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -838,7 +838,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         # Save session files
         session = _PGET('LAST_SESSION')
-        if not session or session is None:
+        if not isinstance(session, basestring):
             session = os.path.join(CONFIG['SESSION_DIR'], u"__default.session")
         _PSET('LAST_SESSION', session)
         self.nb.SaveSessionFile(session)
@@ -960,7 +960,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
     def OnMaximizeEditor(self, evt):
         """Maximize the editor and hide the other panes. If the editor
         is already maximized, it is un-maximized and the other panes are restored  
-        @param event: CommandEvent instance
+        @param evt: CommandEvent instance
 
         """
         paneInfo = self._mgr.GetPane("EditPane")
@@ -1073,6 +1073,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         active_only = [ ID_ZOOM_IN, ID_ZOOM_OUT, ID_ZOOM_NORMAL,
                         ID_JOIN_LINES, ID_CUT_LINE, ID_COPY_LINE, ID_INDENT,
                         ID_UNINDENT, ID_TRANSPOSE, ID_TOGGLECOMMENT,
+                        ID_LINE_MOVE_UP, ID_LINE_MOVE_DOWN,
                         ID_SELECTALL, ID_UNDO, ID_REDO, ID_CUT, ID_COPY,
                         ID_PASTE, ID_LINE_BEFORE, ID_LINE_AFTER, ID_DUP_LINE,
                         ID_PASTE_AFTER, ID_COLUMN_MODE, ID_TOGGLE_FOLD,
@@ -1150,7 +1151,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
     def OnPaneList(self, evt):
         """Navigates through panes
-        @param event: CommandEvent instance
+        @param evt: CommandEvent instance
 
         """
         if evt.GetId() == ID_PANELIST:
