@@ -13,8 +13,8 @@ This class implements Editra's main notebook control.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 59800 2009-03-24 05:47:46Z CJP $"
-__revision__ = "$Revision: 59800 $"
+__svnid__ = "$Id: ed_pages.py 60397 2009-04-26 19:20:56Z CJP $"
+__revision__ = "$Revision: 60397 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -213,6 +213,7 @@ class EdPages(FNB.FlatNotebook):
         super(EdPages, self).AddPage(page, text, select, imgId)
         sel = self.GetSelection()
         self.EnsureVisible(sel)
+        self.UpdateIndexes()
 
     def DocDuplicated(self, path):
         """Check for if the given path is open elswhere and duplicate the
@@ -278,6 +279,11 @@ class EdPages(FNB.FlatNotebook):
         return [(ed_glob.ID_FIND_NEXT, self._searchctrl.OnUpdateFindUI),
                 (ed_glob.ID_FIND_PREVIOUS, self._searchctrl.OnUpdateFindUI)]
 
+    def InsertPage(self, index, page, text, select=True, imageId=-1):
+        """Insert a page into the notebook"""
+        super(EdPages, self).InsertPage(index, page, text, select, imageId)
+        self.UpdateIndexes()
+
     def SaveSessionFile(self, session):
         """Save the current open files to the given session file
         @param session: path to session file
@@ -304,7 +310,6 @@ class EdPages(FNB.FlatNotebook):
 
         """
         self._ses_load = True
-#        files = Profile_Get('LAST_SESSION')
 
         if os.path.exists(session):
             try:
@@ -320,17 +325,18 @@ class EdPages(FNB.FlatNotebook):
 
         # Load and validate file
         try:
-            flist = cPickle.load(f_handle)
-            # TODO: Extend in future to support loading sessions
-            #       for mutiple windows.
-            flist = flist.get('win1', list()) 
-            for item in flist:
-                if type(item) not in (unicode, str):
-                    raise TypeError('Invalid item in unpickled sequence')
-        except (cPickle.UnpicklingError, TypeError), e:
-            dlg.Destroy()
-            return (_('Invalid file'),
-                    _('Selected file is not a valid session file'))
+            try:
+                flist = cPickle.load(f_handle)
+                # TODO: Extend in future to support loading sessions
+                #       for mutiple windows.
+                flist = flist.get('win1', list()) 
+                for item in flist:
+                    if type(item) not in (unicode, str):
+                        raise TypeError('Invalid item in unpickled sequence')
+            except (cPickle.UnpicklingError, TypeError), e:
+                dlg.Destroy()
+                return (_('Invalid file'),
+                        _('Selected file is not a valid session file'))
         finally:
             f_handle.close()
 
@@ -400,7 +406,6 @@ class EdPages(FNB.FlatNotebook):
 
     def OnDocPointerRequest(self, args):
         """Get a buffer that has the same file open as the requested path.
-        @param sender: Notebook that sent the message
         @param args: [sender, path]
         @return: EdEditorView reference or ed_msg.NullValue
 
@@ -502,6 +507,7 @@ class EdPages(FNB.FlatNotebook):
         self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
 
         # Set tab image
+        # TODO: Handle read only images
         self.SetPageImage(self.GetSelection(), str(nbuff.GetLangId()))
 
         # Refocus on selected page
@@ -636,6 +642,7 @@ class EdPages(FNB.FlatNotebook):
                 control.Destroy()
             else:
                 # We where using an existing buffer so reset it
+                "CLEARED IT"
                 control.SetText('')
                 control.SetDocument(ed_txt.EdFile())
                 control.SetSavePoint()
@@ -654,11 +661,6 @@ class EdPages(FNB.FlatNotebook):
         doc = self.control.GetDocument()
         doc.AddModifiedCallback(self.control.FireModified)
 
-        if Profile_Get('SAVE_POS'):
-            pos = self.DocMgr.GetPos(self.control.GetFileName())
-            self.control.GotoPos(pos)
-            self.control.ScrollToColumn(0)
-
         # Add the buffer to the notebook
         if new_pg:
             self.AddPage(self.control, filename)
@@ -666,7 +668,6 @@ class EdPages(FNB.FlatNotebook):
         self.frame.SetTitle(self.control.GetTitleString())
         self.frame.AddFileToHistory(path2file)
         self.SetPageText(self.GetSelection(), filename)
-        self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
 
         # Set tab image
         cpage = self.GetSelection()
@@ -676,13 +677,25 @@ class EdPages(FNB.FlatNotebook):
         else:
             self.SetPageImage(cpage, str(self.control.GetLangId()))
 
+        if Profile_Get('WARN_EOL', default=True):
+            self.control.CheckEOL()
+
+        if not control.IsLoading():
+            self.DoPostLoad()
+
         # Refocus on selected page
         self.GoCurrentPage()
         self.GetTopLevelParent().Thaw()
-        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.control.GetFileName())
+        self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
 
-        if Profile_Get('WARN_EOL', default=True):
-            self.control.CheckEOL()
+    def DoPostLoad(self):
+        """Perform post file open actions"""
+        if Profile_Get('SAVE_POS'):
+            pos = self.DocMgr.GetPos(self.control.GetFileName())
+            self.control.GotoPos(pos)
+            self.control.ScrollToColumn(0)
+
+        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.control.GetFileName())
 
     def GoCurrentPage(self):
         """Move Focus to Currently Selected Page.
@@ -740,6 +753,14 @@ class EdPages(FNB.FlatNotebook):
             txt = ''
         return txt
 
+    def ImageIsReadOnly(self, index):
+        """Does the given page currently have a ReadOnly Image
+        shown on it?
+        @return: bool
+
+        """
+        return self.GetPageImage(index) == self._index[ed_glob.ID_READONLY]
+
     def SetPageText(self, pg_num, txt):
         """Set the pages tab text
         @param pg_num: page index
@@ -959,6 +980,7 @@ class EdPages(FNB.FlatNotebook):
         cpage = self.GetSelection()
         evt.Skip()
         self.LOG("[ed_pages][evt] Closed Page: #%d" % cpage)
+        self.UpdateIndexes()
         ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSED, (self, cpage))
 
     #---- End Event Handlers ----#
@@ -1044,6 +1066,12 @@ class EdPages(FNB.FlatNotebook):
 
         self.Refresh()
 
+    def UpdateIndexes(self):
+        """Update all page indexes"""
+        pages = [self.GetPage(page) for page in xrange(self.GetPageCount())]
+        for idx, page in enumerate(pages):
+            page.SetTabIndex(idx)
+
     def UpdatePageImage(self):
         """Updates the page tab image
         @postcondition: page image is updated to reflect any changes in ctrl
