@@ -7,26 +7,23 @@
 ###############################################################################
 
 """
-Provides various search controls and searching services for finding text in a
-document. The L{TextFinder} is a search service that can be used to search and
-highlight text in a StyledTextCtrl.
+Provides text searching services, utilities, and ui components for searching
+text documents and files.
 
-@summary: Text searching and result ui
+@summary: Text searching and results presentation ui
 
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 59531 2009-03-14 15:16:48Z CJP $"
-__revision__ = "$Revision: 59531 $"
+__svnid__ = "$Id: ed_search.py 60329 2009-04-25 01:53:20Z CJP $"
+__revision__ = "$Revision: 60329 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import os
 import sys
 import re
-import fnmatch
 import types
-from StringIO import StringIO
 import wx
 
 # Local imports
@@ -35,9 +32,9 @@ import ed_txt
 import ed_msg
 import plugin
 import iface
-from util import FileTypeChecker
 from profiler import Profile_Get, Profile_Set
 import eclib
+import ebmlib
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -71,14 +68,22 @@ class SearchController(object):
         self._parent.Bind(eclib.EVT_FIND, self.OnFind)
         self._parent.Bind(eclib.EVT_FIND_NEXT, self.OnFind)
         self._parent.Bind(eclib.EVT_FIND_ALL, self.OnFindAll)
+        self._parent.Bind(eclib.EVT_COUNT, self.OnCount)
         self._parent.Bind(eclib.EVT_REPLACE, self.OnReplace)
         self._parent.Bind(eclib.EVT_REPLACE_ALL, self.OnReplaceAll)
         self._parent.Bind(eclib.EVT_FIND_CLOSE, self.OnFindClose)
         self._parent.Bind(eclib.EVT_OPTION_CHANGED, self.OnOptionChanged)
 
+        # Editra message handlers
+        ed_msg.Subscribe(self._OnShowFindMsg, ed_msg.EDMSG_FIND_SHOW_DLG)
+
+    def __del__(self):
+        """Cleanup message handlers"""
+        ed_msg.Unsubscribe(self._OnShowFindMsg)
+
     def _CreateNewDialog(self, e_id):
         """Create and set the controlers find dialog
-        @param eid: Dialog Type Id
+        @param e_id: Dialog Type Id
 
         """
         if e_id == ed_glob.ID_FIND_REPLACE:
@@ -109,6 +114,41 @@ class SearchController(object):
 
         return dlg
 
+    def _OnShowFindMsg(self, msg):
+        """Message handler for clients to request and setup the find dialog
+        with.
+        @param msg: dict(mainw, lookin, findtxt)
+
+        """
+        data = msg.GetData()
+        if data.get('mainw', None) == self._parent.GetTopLevelParent():
+
+            if 'findtxt' in data:
+                self.SetQueryString(data.get('findtxt'))
+            else:
+                query = self.GetClientString()
+                if len(query):
+                    self.SetQueryString(query)
+
+            # Dialog is not currently open
+            if self._finddlg is None:
+                self._finddlg = self._CreateNewDialog(ed_glob.ID_FIND)
+                if self._finddlg is None:
+                    return
+                self._finddlg.SetTransparent(240)
+    #            self._finddlg.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
+                self._finddlg.Show()
+            else:
+                # Dialog is open already so just update it
+                self._UpdateDialogState(ed_glob.ID_FIND)
+                self._finddlg.Raise()
+
+            if 'lookin' in data:
+                self._finddlg.SetLookinPath(data.get('lookin'))
+            self._finddlg.SetFocus()
+        else:
+            return
+
     def _UpdateDialogState(self, e_id):
         """Update the state of the existing dialog"""
         if self._finddlg is None:
@@ -181,6 +221,27 @@ class SearchController(object):
         else:
             evt.Skip()
 
+    def OnCount(self, evt):
+        """Count the number of matches"""
+        stc = self._stc()
+
+        # Create the search engine
+        query = evt.GetFindString()
+        engine = ebmlib.SearchEngine(query, evt.IsRegEx(),
+                                     True, evt.IsMatchCase(), evt.IsWholeWord())
+        engine.SetSearchPool(stc.GetTextRaw())
+
+        matches = engine.FindAll()
+        if matches:
+            count = len(matches)
+        else:
+            count = 0
+
+        rmap = dict(term=query, count=count)
+        wx.MessageBox(_("The search term \'%(term)s\' was found %(count)d times.") % rmap,
+                      _("Find Count"),
+                      wx.ICON_INFORMATION|wx.OK)
+
     def OnFind(self, evt, findnext=False):
         """Do an incremental search in the currently buffer
         @param evt: EVT_FIND, EVT_FIND_NEXT
@@ -206,8 +267,9 @@ class SearchController(object):
 
         # Create the search engine
         isdown = not evt.IsUp()
-        engine = SearchEngine(data.GetFindString(), evt.IsRegEx(),
-                              isdown, evt.IsMatchCase(), evt.IsWholeWord())
+        engine = ebmlib.SearchEngine(data.GetFindString(), evt.IsRegEx(),
+                                     isdown, evt.IsMatchCase(),
+                                     evt.IsWholeWord())
         engine.SetResultFormatter(FormatResult)
         self._engine = engine
 
@@ -296,7 +358,7 @@ class SearchController(object):
 
     def OnFindAll(self, evt):
         """Find all results for the given query and display results in a
-        L{FindResultsScreen} in the Shelf.
+        L{SearchResultScreen} in the Shelf.
 
         """
         smode = evt.GetSearchType()
@@ -305,8 +367,8 @@ class SearchController(object):
             return
 
         # Create a new search engine object
-        engine = SearchEngine(query, evt.IsRegEx(), True,
-                              evt.IsMatchCase(), evt.IsWholeWord())
+        engine = ebmlib.SearchEngine(query, evt.IsRegEx(), True,
+                                     evt.IsMatchCase(), evt.IsWholeWord())
         engine.SetResultFormatter(FormatResult)
 
         # Send the search function over to any interested parties that wish
@@ -319,7 +381,8 @@ class SearchController(object):
                                    (engine.SearchInFile, [fname,], dict()))
             else:
                 engine.SetSearchPool(stc.GetTextRaw())
-                ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH, (engine.FindAllLines,))
+                ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH,
+                                   (engine.FindAllLines,))
         elif smode == eclib.LOCATION_OPEN_DOCS:
             files = [fname.GetFileName()
                      for fname in self._parent.GetTextControls()]
@@ -412,8 +475,8 @@ class SearchController(object):
         """
         smode = evt.GetSearchType()
         rstring = evt.GetReplaceString()
-        engine = SearchEngine(evt.GetFindString(), evt.IsRegEx(),
-                              True, evt.IsMatchCase(), evt.IsWholeWord())
+        engine = ebmlib.SearchEngine(evt.GetFindString(), evt.IsRegEx(),
+                                     True, evt.IsMatchCase(), evt.IsWholeWord())
         engine.SetResultFormatter(FormatResult)
 
         results = 0
@@ -518,7 +581,7 @@ class SearchController(object):
         for match in reversed(matches):
             start, end = match.span()
             if isregex:
-                value = match.expand(rstring)
+                value = match.expand(rstring.encode('utf-8')).decode('utf-8')
             else:
                 value = rstring
             stc.SetTargetStart(start)
@@ -564,354 +627,6 @@ class SearchController(object):
 
 #-----------------------------------------------------------------------------#
 
-class SearchEngine(object):
-    """Text Search Engine
-    All Search* methods are iterable generators
-    All Find* methods do a complete search and return the match collection
-    @summary: Text Search Engine
-    @todo: Add file filter support
-
-    """
-    def __init__(self, query, regex=True, down=True,
-                  matchcase=True, wholeword=False):
-        """Initialize a search engine object
-        @param query: search string
-        @keyword regex: Is a regex search
-        @keyword down: Search down or up
-        @keyword matchcase: Match case
-        @keyword wholeword: Match whole word
-
-        """
-        object.__init__(self)
-
-        # Attributes
-        self._isregex = regex
-        self._next = down
-        self._matchcase = matchcase
-        self._wholeword = wholeword
-        self._unicode = False
-        self._query = query
-        self._regex = u''
-        self._pool = u''
-        self._lmatch = None             # Last match object
-        self._filters = None            # File Filters
-        self._formatter = lambda f, l, m: u"%s %d: %s" % (f, l+1, m)
-        self._CompileRegex()
-
-    def _CompileRegex(self):
-        """Prepare and compile the regex object based on the current state
-        and settings of the engine.
-        @postcondition: the engines regular expression is created
-
-        """
-        tmp = self._query
-        if not self._isregex:
-            tmp = re.escape(tmp)
-
-        if self._wholeword:
-            tmp = "\\b%s\\b" % tmp
-
-        flags = re.MULTILINE
-
-        if not self._matchcase:
-            flags |= re.IGNORECASE
-
-        if self._unicode:
-            flags |= re.UNICODE
-
-        try:
-            self._regex = re.compile(tmp, flags)
-        except:
-            self._regex = None
-
-    def Find(self, spos=0):
-        """Find the next match based on the state of the search engine
-        @keyword spos: search start position
-        @return: tuple (match start pos, match end pos) or None if no match
-        @prerequisite: L{SetSearchPool} has been called to set search string
-
-        """
-        if self._regex is None:
-            return None
-
-        if self._next:
-            return self.FindNext(spos)
-        else:
-            if spos == 0:
-                spos = -1
-            return self.FindPrev(spos)
-
-    def FindAll(self):
-        """Find all the matches in the current context
-        @return: list of tuples [(start1, end1), (start2, end2), ]
-
-        """
-        if self._regex is None:
-            return list()
-
-        matches = [match for match in self._regex.finditer(self._pool)]
-        return matches
-
-    def FindAllLines(self):
-        """Find all the matches in the current context
-        @return: list of strings
-
-        """
-        rlist = list()
-        if self._regex is None:
-            return rlist
-
-        for lnum, line in enumerate(StringIO(self._pool)):
-            if self._regex.search(line) is not None:
-                rlist.append(self._formatter(_("Untitled"), lnum, line))
-
-        return rlist
-
-    def FindNext(self, spos=0):
-        """Find the next match of the query starting at spos
-        @keyword spos: search start position in string
-        @return: tuple (match start pos, match end pos) or None if no match
-        @prerequisite: L{SetSearchPool} has been called to set the string to
-                       search in.
-
-        """
-        if self._regex is None:
-            return None
-
-        if spos < len(self._pool):
-            match = self._regex.search(self._pool[spos:])
-            if match is not None:
-                self._lmatch = match
-                return match.span()
-        return None
-
-    def FindPrev(self, spos=-1):
-        """Find the previous match of the query starting at spos
-        @param pool: string to search in
-        @keyword spos: search start position in string
-        @return: tuple (match start pos, match end pos)
-
-        """
-        if self._regex is None:
-            return None
-
-        if spos+1 < len(self._pool):
-            matches = [match for match in self._regex.finditer(self._pool[:spos])]
-            if len(matches):
-                lmatch = matches[-1]
-                self._lmatch = lmatch
-                return (lmatch.start(), lmatch.end())
-        return None
-
-    def GetLastMatch(self):
-        """Get the last found match object from the previous L{FindNext} or
-        L{FindPrev} action.
-        @return: match object or None
-
-        """
-        return self._lmatch
-
-    def GetOptionsString(self):
-        """Get a string describing the search engines options"""
-        rstring = u"\"%s\" [ " % self._query
-        for desc, attr in ((_("regex: %s"), self._isregex),
-                           (_("match case: %s"), self._matchcase),
-                           (_("whole word: %s"), self._wholeword)):
-            if attr:
-                rstring += (desc % u"on; ")
-            else:
-                rstring += (desc % u"off; ")
-        rstring += u"]"
-
-        return rstring
-
-    def GetQuery(self):
-        """Get the raw query string used by the search engine
-        @return: string
-
-        """
-        return self._query
-
-    def GetQueryObject(self):
-        """Get the regex object used for the search. Will return None if
-        there was an error in creating the object.
-        @return: pattern object
-
-        """
-        return self._regex
-
-    def GetSearchPool(self):
-        """Get the search pool string for this L{SearchEngine}.
-        @return: string
-
-        """
-        return self._pool
-
-    def IsMatchCase(self):
-        """Is the engine set to a case sensitive search
-        @return: bool
-
-        """
-        return self._matchcase
-
-    def IsRegEx(self):
-        """Is the engine searching with the query as a regular expression
-        @return: bool
-
-        """
-        return self._isregex
-
-    def IsWholeWord(self):
-        """Is the engine set to search for wholeword matches
-        @return: bool
-
-        """
-        return self._wholeword
-
-    def SearchInBuffer(self, sbuffer):
-        """Search in the buffer
-        @param sbuffer: buffer like object
-        @todo: implement
-
-        """
-        raise NotImplementedError
-
-    def SearchInDirectory(self, directory, recursive=True):
-        """Search in all the files found in the given directory
-        @param directory: directory path
-        @keyword recursive: search recursivly
-
-        """
-        if self._regex is None:
-            return
-
-        # Get all files in the directories
-        paths = [os.path.join(directory, fname)
-                for fname in os.listdir(directory) if not fname.startswith('.')]
-
-        # Filter out files that don't match the current filter(s)
-        if self._filters is not None and len(self._filters):
-            filtered = list()
-            for fname in paths:
-                if os.path.isdir(fname):
-                    filtered.append(fname)
-                    continue
-
-                for pat in self._filters:
-                    if fnmatch.fnmatch(fname, pat):
-                        filtered.append(fname)
-            paths = filtered
-
-        # Begin searching in the paths
-        for path in paths:
-            if recursive and os.path.isdir(path):
-                # Recursive call to decend into directories
-                for match in self.SearchInDirectory(path, recursive):
-                    yield match
-            else:
-                for match in self.SearchInFile(path):
-                    yield match
-        return
-
-    def SearchInFile(self, fname):
-        """Search in a file for all lines with matches of the set query and
-        yield the results as they are found.
-        @param fname: filename
-        @todo: unicode handling
-
-        """
-        if self._regex is None:
-            return
-
-        fchecker = FileTypeChecker()
-        if fchecker.IsReadableText(fname):
-            try:
-                fobj = open(fname, 'rb')
-            except (IOError, OSError):
-                return
-            else:
-                # Special token to signify start of a search
-                yield (None, fname)
-
-            for lnum, line in enumerate(fobj):
-                if self._regex.search(line) is not None:
-                    yield self._formatter(fname, lnum, line)
-            fobj.close()
-        return
-
-    def SearchInFiles(self, flist):
-        """Search in a list of files and yield results as they are found.
-        @param flist: list of file names
-
-        """
-        if self._regex is None:
-            return
-
-        for fname in flist:
-            for match in self.SearchInFile(fname):
-                yield match
-        return
-
-    def SearchInString(self, sstring, startpos=0):
-        """Search in a string
-        @param sstring: string to search in
-        @keyword startpos: search start position
-
-        """
-        raise NotImplementedError
-
-    def SetFileFilters(self, filters):
-        """Set the file filters to specify what type of files to search in
-        the filter should be a list of wild card patterns to match.
-        @param filters: list of strings ['*.py', '*.pyw']
-
-        """
-        self._filters = filters
-
-    def SetFlags(self, isregex=None, matchcase=None, wholeword=None, down=None):
-        """Set the search engine flags. Leaving the parameter set to None
-        will not change the flag. Setting it to non None will change the value.
-        @keyword isregex: is regex search
-        @keyword matchcase: matchcase search
-        @keyword wholeword: wholeword search
-        @keyword down: search down or up
-
-        """
-        for attr, val in (('_isregex', isregex), ('_matchcase', matchcase),
-                          ('_wholeword', wholeword), ('_next', down)):
-            if val is not None:
-                setattr(self, attr, val)
-        self._CompileRegex()
-
-    def SetResultFormatter(self, funct):
-        """Set the result formatter function
-        @param funct: callable(filename, linenum, matchstr)
-
-        """
-        assert callable(funct)
-        self._formatter = funct
-
-    def SetSearchPool(self, pool):
-        """Set the search pool used by the Find methods
-        @param pool: string to search in
-
-        """
-        del self._pool
-        self._pool = pool
-        if isinstance(self._pool, types.UnicodeType):
-            self._unicode = True
-            self._CompileRegex()
-
-    def SetQuery(self, query):
-        """Set the search query
-        @param query: string
-
-        """
-        self._query = query
-        self._CompileRegex()
-
-#-----------------------------------------------------------------------------#
-
 def FormatResult(fname, lnum, match):
     """Format the search result string
     @return: string
@@ -936,7 +651,7 @@ def FormatResult(fname, lnum, match):
 class EdSearchCtrl(wx.SearchCtrl):
     """Creates a simple search control for use in the toolbar
     or a statusbar and the such. Supports incremental search,
-    and uses L{TextFinder} to do the actual searching of the
+    and uses L{SearchController} to do the actual searching of the
     document.
 
     """
@@ -1498,9 +1213,9 @@ class SearchResultScreen(eclib.ControlBox):
 
 class SearchResultList(eclib.OutputBuffer):
     """Outputbuffer for listing matching lines from the search results that
-    a L{SearchEngine} dispatches. The matching lines are turned into hotspots
-    that allow them to be clicked on for instant navigation to the matching
-    line.
+    a L{ebmlib.SearchEngine} dispatches. The matching lines are turned into
+    hotspots that allow them to be clicked on for instant navigation to the
+    matching line.
 
     """
     STY_SEARCH_MATCH = eclib.OPB_STYLE_MAX + 1
@@ -1604,7 +1319,6 @@ class SearchResultList(eclib.OutputBuffer):
         @param fname: File name to open, relative paths will be converted to abs
                       paths.
         @param line: Line number to set the cursor to after opening the file
-        @param mainw: MainWindow instance to open the file in
 
         """
         mainw = wx.GetApp().GetActiveWindow()
