@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 59714 2009-03-22 01:31:07Z CJP $"
-__revision__ = "$Revision: 59714 $"
+__svnid__ = "$Id: ed_stc.py 60360 2009-04-25 18:06:21Z CJP $"
+__revision__ = "$Revision: 60360 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -38,6 +38,7 @@ import ed_msg
 import ed_mdlg
 import ed_txt
 from ed_keyh import KeyHandler, ViKeyHandler
+import ebmlib
 
 #-------------------------------------------------------------------------#
 # Globals
@@ -87,6 +88,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
         # Attributes
         self.LOG = wx.GetApp().GetLog()
+        self._loading = None
         self.key_handler = KeyHandler(self)
 
         # Macro Attributes
@@ -98,7 +100,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
                             autoindent=_PGET('AUTO_INDENT'),
                             brackethl=_PGET('BRACKETHL'),
                             folding=_PGET('CODE_FOLD'),
-                            highlight=_PGET('SYNTAX'))
+                            highlight=_PGET('SYNTAX'),
+                            autobkup=_PGET('AUTOBACKUP'))
 
         # Set Default Styles used by all documents
         self.Configure()
@@ -215,18 +218,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
         @postcondition: a new line is added to the document
 
         """
-        line = self.LineFromPosition(self.GetCurrentPos())
         if before:
-            line = max(line - 1, 0)
-
-        if line or not before:
-            pos = self.GetLineEndPosition(line)
-            curr = len(self.GetEOLChar())
-        else:
-            pos = 0
-            curr = 0
-        self.InsertText(pos, self.GetEOLChar())
-        self.GotoPos(pos + curr)
+            self.LineUp()
+        self.LineEnd()
+        self.AutoIndent()
+        # We need LineEnd first because AutoIdent works as if
+        # we pressed enter at caret position
 
     def Bookmark(self, action):
         """Handles bookmark actions
@@ -455,12 +452,14 @@ class EditraStc(ed_basestc.EditraBaseStc):
             column = linelen
         self.SetCurrentPos(lstart + column)
 
-    def GotoIndentPos(self, line):
+    def GotoIndentPos(self, line=None):
         """Move the caret to the end of the indentation
         on the given line.
         @param line: line to go to
 
         """
+        if line is None:
+            line = self.GetCurrentLine()
         self.GotoPos(self.GetLineIndentPosition(line))
 
     def DeleteForward(self):
@@ -671,10 +670,38 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
     def OnLoadProgress(self, evt):
         """Recieves file loading events from asynchronous file loading"""
-        if evt.HasText():
+        pid = self.GetTopLevelParent().GetId()
+        if evt.GetState() == ed_txt.FL_STATE_READING:
+            if evt.HasText():
+                # TODO: gauge gauge updates working properly
+#                sb = self.GetTopLevelParent().GetStatusBar()
+#                gauge = sb.GetGauge()
+#                gauge.SetValue(evt.GetProgress())
+#                gauge.Show()
+#                gauge.ProcessPendingEvents()
+#                sb.ProcessPendingEvents()
+                self.SetReadOnly(False)
+                self.AppendText(evt.GetValue())
+                self.SetReadOnly(True)
+                # wx.GetApp().Yield(True) # Too slow on windows...
+        elif evt.GetState() == ed_txt.FL_STATE_END:
             self.SetReadOnly(False)
-            #self.AppendText(evt.GetValue())
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE, (pid, 0, 0))
+            self.SetSavePoint()
+            self.SetUndoCollection(True)
+            del self._loading
+            self._loading = None
+            parent = self.GetParent()
+            if hasattr(parent, 'DoPostLoad'):
+                parent.DoPostLoad()
+        elif evt.GetState() == ed_txt.FL_STATE_START:
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW, (pid, True))
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE, (pid, 0, self.File.GetSize()))
             self.SetReadOnly(True)
+            self.SetUndoCollection(False)
+        elif evt.GetState() == ed_txt.FL_STATE_ABORTED:
+            self.SetReadOnly(False)
+            self.ClearAll()
 
     def OnUpdateUI(self, evt):
         """Check for matching braces
@@ -842,6 +869,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
                   ed_glob.ID_SYNTAX : self.SyntaxOnOff,
                   ed_glob.ID_UNINDENT : self.BackTab,
                   ed_glob.ID_TRANSPOSE : self.LineTranspose,
+                  ed_glob.ID_LINE_MOVE_UP : self.LineMoveUp,
+                  ed_glob.ID_LINE_MOVE_DOWN : self.LineMoveDown,
                   ed_glob.ID_SELECTALL: self.SelectAll,
                   ed_glob.ID_FOLDING : self.FoldingOnOff,
                   ed_glob.ID_SHOW_LN : self.ToggleLineNumbers,
@@ -1064,6 +1093,13 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         return self._config['highlight']
 
+    def IsLoading(self):
+        """Is a background thread loading the text into the file
+        @return: bool
+
+        """
+        return self._loading is not None
+
     def IsRecording(self):
         """Returns whether the control is in the middle of recording
         a macro or not.
@@ -1115,6 +1151,26 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetTargetEnd(self.GetLineEndPosition(eline))
         self.ReplaceTarget(u' '.join(lines))
 
+    def LineMoveUp(self):
+        """Move the current line up"""
+        linenum = self.GetCurrentLine()
+        if linenum > 0 :
+            self.BeginUndoAction()
+            self.LineTranspose()
+            self.LineUp()
+            self.EndUndoAction()
+
+    def LineMoveDown(self):
+        """Move the current line down"""
+        linenum = self.GetCurrentLine()
+        col = self.GetColumn(self.GetCurrentPos())
+        if linenum < self.GetLineCount() - 1:
+            self.BeginUndoAction()
+            self.LineDown()
+            self.LineTranspose()
+            self.GotoColumn(col)
+            self.EndUndoAction()
+
     def LineTranspose(self):
         """Switch the current line with the previous one
         @note: overrides base stc method to do transpose in single undo action
@@ -1137,7 +1193,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
     def SetEOLMode(self, mode):
         """Sets the EOL mode from a string descript
-        @param mode_str: eol mode to set
+        @param mode: eol mode to set
         @note: overrides StyledTextCtrl.SetEOLMode
 
         """
@@ -1320,6 +1376,123 @@ class EditraStc(ed_basestc.EditraBaseStc):
             lineNum = self.GetCurrentLine()
         super(EditraStc, self).ToggleFold(lineNum)
 
+    def MoveCurrentPos(self, offset, extend=False):
+        """Move caret by the given offset
+        @note: only use it for movement within a line
+
+        """
+        if offset > 0:
+            step = self.CharRight #XXX breaks RTL, but is RTL supported?
+            if extend:
+                step = self.CharRightExtend
+        else:
+            step = self.CharLeft
+            if extend:
+                step = self.CharLeftExtend
+
+        #XXX is there a better reliable way than repeating basic movements?
+        repeat = abs(offset)
+        for i in range(repeat):
+            step()
+
+    def _FindChar(self, char, repeat=1, reverse=False, extra_offset=0):
+        """Find the position of the next (ith) 'char' character
+        on the current line
+
+        @note used by vim motions for finding a character on a line (f,F,t,T)
+        @param char: the char to be found
+        @keyword repeat: how many times to repeat the serach
+        @keyword reverse: whether to search backwards
+        @keyword extra_offset: extra offset to be applied to the return value
+
+        @return: offset from caret position
+
+        """
+        text, pos = self.GetCurLine()
+        oldpos = pos
+        if not reverse:
+            for i in range(repeat):
+                pos = text.find(char, pos+1) # pos is on the char itself
+                if pos == -1:
+                    break # -1 + 1 = 0
+        else:
+            for i in range(repeat):
+                pos = text.rfind(char, 0, pos)
+                if pos == -1:
+                    break # TEST not sure but we need this as well
+
+        # if pos == -1 then the char was not found
+        # but offset could be arbitrary
+        # (although should never be other than 0 or -1) so do a bound check
+        newpos = pos + extra_offset
+        if newpos not in range(len(text)):
+            return 0
+
+        return newpos - oldpos;
+
+    def FindNextChar(self, char, repeat=1, extend=False):
+        """Move caret to the next (ith) occurance of char on the current line
+        @note: This is a vim motion
+        @keyword repeat: int determining ith occurance to move to
+        @keyword extend: whether to extend selection or not
+
+        """
+        offset = self._FindChar(char, repeat)
+        self.MoveCurrentPos(offset, extend)
+
+    def FindPrevChar(self, char, repeat=1, extend=False):
+        """Move caret to the previous (ith) occurance of char on the current line
+        @note: This is a vim motion
+        @keyword repeat: int determining ith occurance to move to
+        @keyword extend: whether to extend selection or not
+
+        """
+
+        offset = self._FindChar(char, repeat, reverse=True)
+        self.MoveCurrentPos(offset, extend)
+
+    def FindTillNextChar(self, char, repeat=1, extend=False):
+        """Move caret until right before the next ith occurance of char
+
+        @note: This is a vim motion
+        @keyword repeat: int determining ith occurance to move to
+        @keyword extend: whether to extend selection or not
+
+        """
+        offset = self._FindChar(char, repeat, extra_offset=-1)
+        self.MoveCurrentPos(offset, extend)
+
+    def FindTillPrevChar(self, char, repeat=1, extend=False):
+        """Move caret until right before the previous ith occurance of char
+
+        @note: This is a vim motion
+        @keyword repeat: int determining ith occurance to move to
+        @keyword extend: whether to extend selection or not
+
+        """
+        offset = self._FindChar(char, repeat, reverse=True, extra_offset=-1)
+        self.MoveCurrentPos(offset, extend)
+
+
+    def GetIdentifierUnderCursor(self):
+        """Returns the identifier under the cursor (if any)"""
+        line, pos = self.GetCurLine()
+        while pos > 0: # rewind to first char of identifier
+            c = line[pos-1]
+            if c == '_' or c.isalnum():
+               pos = pos - 1
+            else:
+                break
+
+        start = pos
+        while pos < len(line): # find end of identifier
+            c = line[pos]
+            if c == '_' or c.isalnum():
+               pos = pos + 1
+            else:
+                break
+        return line[start:pos]
+
     def WordLeft(self):
         """Move caret to begining of previous word
         @note: override builtin to include extra characters in word
@@ -1442,6 +1615,27 @@ class EditraStc(ed_basestc.EditraBaseStc):
             super(EditraStc, self).WordRightExtend()
         self.SetWordChars('')
 
+    def LoadFile(self, path):
+        """Load the file at the given path into the buffer. Returns
+        True if no errors and False otherwise. To retrieve the errors
+        check the last error that was set in the file object returned by
+        L{GetDocument}.
+        @param path: path to file
+
+        """
+        fsize = ebmlib.GetFileSize(path)
+        tlw = self.GetTopLevelParent()
+        pid = tlw.GetId()
+
+        if fsize < 1048576: # 1MB
+            return super(EditraStc, self).LoadFile(path)
+        else:
+            ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENING, path)
+            self.file.SetPath(path)
+            self.file.ReadAsync(self)
+            self._loading = wx.BusyCursor()
+            return True
+
     def ReloadFile(self):
         """Reloads the current file, returns True on success and
         False if there is a failure.
@@ -1455,8 +1649,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 self.BeginUndoAction()
                 marks = self.GetBookmarks()
                 cpos = self.GetCurrentPos()
+                # TODO: Handle async re-loads of large files
                 self.SetText(self.File.Read())
-                self.SetModTime(util.GetFileModTime(cfile))
+                self.SetModTime(ebmlib.GetFileModTime(cfile))
                 for mark in marks:
                     self.MarkerAdd(mark, ed_basestc.MARK_MARGIN)
                 self.EndUndoAction()
@@ -1529,7 +1724,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
         if result:
             self.SetSavePoint()
-            self.SetModTime(util.GetFileModTime(path))
+            self.SetModTime(ebmlib.GetFileModTime(path))
             self.File.FireModified()
             self.SetFileName(path)
 
