@@ -20,8 +20,8 @@ U{http://editra.org/?page=docs&doc=ess_spec}.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 57230 2008-12-10 05:40:35Z CJP $"
-__revision__ = "$Revision: 57230 $"
+__svnid__ = "$Id: ed_style.py 59503 2009-03-13 02:21:14Z CJP $"
+__revision__ = "$Revision: 59503 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -33,6 +33,7 @@ import wx
 import ed_glob
 import util
 from profiler import Profile_Get, Profile_Set
+import eclib
 
 # Globals
 STY_ATTRIBUTES     = u"face fore back size modifiers"
@@ -478,7 +479,7 @@ class StyleMgr(object):
             fore = u"#000000"
 
         if not as_hex:
-            rgb = util.HexToRGB(fore[1:])
+            rgb = eclib.HexToRGB(fore[1:])
             fore = wx.Colour(red=rgb[0], green=rgb[1], blue=rgb[2])
         return fore
 
@@ -504,7 +505,7 @@ class StyleMgr(object):
             back = u"#FFFFFF"
 
         if not as_hex:
-            rgb = util.HexToRGB(back[1:])
+            rgb = eclib.HexToRGB(back[1:])
             back = wx.Colour(red=rgb[0], green=rgb[1], blue=rgb[2])
         return back
 
@@ -586,7 +587,7 @@ class StyleMgr(object):
             style = Profile_Get('SYNTHEME', 'str').lower()
 
         # Get Correct Filename if it exists
-        for sheet in util.GetResourceFiles('styles', False, True, title=False):
+        for sheet in util.GetResourceFiles(u'styles', False, True, title=False):
             if sheet.lower() == style.lower():
                 style = sheet
                 break
@@ -943,7 +944,7 @@ class StyleMgr(object):
 
         """
         self.StyleClearAll()
-        self.SetCaretForeground(wx.NamedColor("black"))
+        self.SetCaretForeground(wx.BLACK)
         self.Colourise(0, -1)
 
     def UpdateAllStyles(self, spec_style=None):
@@ -965,6 +966,7 @@ class StyleMgr(object):
         """
         self.StyleDefault()
         self.SetMargins(4, 0)
+
         # Global default styles for all languages
         self.StyleSetSpec(0, self.GetStyleByName('default_style'))
         self.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, \
