@@ -16,8 +16,8 @@ and setting of the program by setting values in the Profile.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 59324 2009-03-04 22:44:44Z CJP $"
-__revision__ = "$Revision: 59324 $"
+__svnid__ = "$Id: prefdlg.py 60365 2009-04-25 23:49:16Z CJP $"
+__revision__ = "$Revision: 60365 $"
 
 #----------------------------------------------------------------------------#
 # Dependancies
@@ -293,9 +293,13 @@ class GeneralPanel(wx.Panel, PreferencesPanelBase):
         splash_cb = wx.CheckBox(self, ed_glob.ID_APP_SPLASH,
                                 _("Show Splash Screen"))
         splash_cb.SetValue(Profile_Get('APPSPLASH'))
-        chk_update = wx.CheckBox(self, ed_glob.ID_PREF_CHKUPDATE,
-                                 _("Check for updates on startup"))
-        chk_update.SetValue(Profile_Get('CHECKUPDATE'))
+
+        # Only enable update option if user has access to install directory
+        isadmin = os.access(ed_glob.CONFIG['INSTALL_DIR'], os.R_OK|os.W_OK)
+        if isadmin:
+            chk_update = wx.CheckBox(self, ed_glob.ID_PREF_CHKUPDATE,
+                                     _("Check for updates on startup"))
+            chk_update.SetValue(Profile_Get('CHECKUPDATE'))
 
         # File settings
         fhsizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -362,9 +366,12 @@ class GeneralPanel(wx.Panel, PreferencesPanelBase):
                        ((5, 5),), (psizer, 0),
                        ((5, 5),), (reporter_cb, 0),
                        ((5, 5),), (sess_cb, 0),
-                       ((5, 5),), (splash_cb, 0),
-                       ((5, 5),), (chk_update, 0),
-                       ((5, 5),), ((5, 5),),
+                       ((5, 5),), (splash_cb, 0)])
+
+        if isadmin:
+            sizer.AddMany([((5, 5),), (chk_update, 0)])
+
+        sizer.AddMany([((5, 5),), ((5, 5),),
                        (wx.StaticText(self, label=_("File Settings") + u": "),
                         0, wx.ALIGN_CENTER_VERTICAL), (enc_sz, 0),
                        ((5, 5),), (fhsizer, 0),
@@ -597,7 +604,7 @@ class DocGenPanel(wx.Panel):
 
     @staticmethod
     def OnFontChange(evt):
-        """Handles L{ecpickers.EVT_FONT_CHANGED} from the font controls"""
+        """Handles L{eclib.EVT_FONT_CHANGED} from the font controls"""
         e_id = evt.GetId()
         if e_id in [DocGenPanel.ID_FONT_PICKER, DocGenPanel.ID_FONT_PICKER2]:
             font = evt.GetValue()
@@ -717,13 +724,21 @@ class DocCodePanel(wx.Panel):
         # Input Helpers
         comp_cb = wx.CheckBox(self, ed_glob.ID_AUTOCOMP, _("Auto-Completion"))
         comp_cb.SetValue(Profile_Get('AUTO_COMP'))
+        compex_cb = wx.CheckBox(self, ed_glob.ID_PREF_AUTOCOMPEX,
+                                _("Extended Auto-Comp"))
+        compex_cb.SetValue(Profile_Get('AUTO_COMP_EX'))
+        compex_cb.Enable(comp_cb.GetValue())
+        compex_cb.SetToolTipString(_("Warning suggestions will include"
+                                     " context insensitive results"))
+        compex_sz = wx.BoxSizer(wx.HORIZONTAL)
+        compex_sz.AddMany([((16, -1), 0), (compex_cb, 0)])
         ai_cb = wx.CheckBox(self, ed_glob.ID_AUTOINDENT, _("Auto-Indent"))
         ai_cb.SetValue(Profile_Get('AUTO_INDENT'))
         vi_cb = wx.CheckBox(self, ed_glob.ID_VI_MODE, _("Enable Vi Emulation"))
         vi_cb.SetValue(Profile_Get('VI_EMU'))
 
         # Layout the controls
-        sizer = wx.FlexGridSizer(13, 2, 5, 5)
+        sizer = wx.FlexGridSizer(14, 2, 5, 5)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("General") + u": "),
                         0, wx.ALIGN_CENTER_VERTICAL), (dlex_sz, 0),
@@ -736,6 +751,7 @@ class DocCodePanel(wx.Panel):
                        ((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Input Helpers") + u": "),
                         0), (comp_cb, 0),
+                       ((5, 5), 0), (compex_sz, 0),
                        ((5, 5), 0), (ai_cb, 0),
                        ((5, 5), 0), (vi_cb, 0),
                        ((10, 10), 0), ((10, 10), 0)])
@@ -753,7 +769,8 @@ class DocCodePanel(wx.Panel):
                     ed_glob.ID_INDENT_GUIDES, ed_glob.ID_FOLDING,
                     ed_glob.ID_AUTOCOMP, ed_glob.ID_AUTOINDENT,
                     ed_glob.ID_PREF_EDGE, ed_glob.ID_VI_MODE,
-                    ed_glob.ID_PREF_DLEXER, ed_glob.ID_HLCARET_LINE):
+                    ed_glob.ID_PREF_DLEXER, ed_glob.ID_HLCARET_LINE,
+                    ed_glob.ID_PREF_AUTOCOMPEX):
 
             e_val = evt.GetEventObject().GetValue()
 
@@ -761,13 +778,22 @@ class DocCodePanel(wx.Panel):
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_val)
 
             # Make ui adjustments
+            meth = None
+            args = list()
             if e_id == ed_glob.ID_SHOW_EDGE:
                 spin = self.FindWindowById(ed_glob.ID_PREF_EDGE)
                 if spin is not None:
                     spin.Enable(e_val)
+            elif e_id == ed_glob.ID_AUTOCOMP:
+                cbox = self.FindWindowById(ed_glob.ID_PREF_AUTOCOMPEX)
+                if cbox is not None:
+                    cbox.Enable(e_val)
+
+            if e_id == ed_glob.ID_PREF_AUTOCOMPEX:
+                meth = 'ConfigureAutoComp'
 
             # Inform views of preference changes
-            wx.CallLater(25, DoUpdates)
+            wx.CallLater(25, DoUpdates, meth, args)
         else:
             evt.Skip()
 
@@ -1095,7 +1121,11 @@ class NetworkPanel(wx.Panel, PreferencesPanelBase):
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         nbook = wx.Notebook(self)
         nbook.AddPage(NetConfigPage(nbook), _("Configuration"))
-        nbook.AddPage(UpdatePage(nbook), _("Update"))
+
+        # Only show update page if user has access to do installs
+        if os.access(ed_glob.CONFIG['INSTALL_DIR'], os.R_OK|os.W_OK):
+            nbook.AddPage(UpdatePage(nbook), _("Update"))
+
         sizer.AddMany([((10, 10), 0), (nbook, 1, wx.EXPAND), ((10, 10), 0)])
         msizer = wx.BoxSizer(wx.VERTICAL)
         msizer.AddMany([(sizer, 1, wx.EXPAND), ((10, 10), 0)])
@@ -1915,7 +1945,7 @@ def GetPrintModeStrings():
             _('Inverse'),           # PRINT_INVERSE
             _('Normal')]            # PRINT_NORMAL
 
-def DoUpdates():
+def DoUpdates(meth=None, args=list()):
     """Update all open text controls"""
     for mainw in wx.GetApp().GetMainWindows():
-        mainw.nb.UpdateTextControls()
+        mainw.nb.UpdateTextControls(meth, args)
