@@ -20,8 +20,8 @@ happen during run time.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: profiler.py 59339 2009-03-05 15:31:32Z CJP $"
-__revision__ = "$Revision: 59339 $"
+__svnid__ = "$Id: profiler.py 60381 2009-04-26 05:53:19Z CJP $"
+__revision__ = "$Revision: 60381 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -42,7 +42,9 @@ _DEFAULTS = {
            'ALPHA'      : 255,              # Transparency level (0-255)
            'AALIASING'  : False,            # Use Anti-Aliasing if availble
            'APPSPLASH'  : True,             # Show splash at startup
+           'AUTOBACKUP' : False,            # Automatically backup files
            'AUTO_COMP'  : True,             # Use Auto-comp if available
+           'AUTO_COMP_EX' : False,          # Use extended autocompletion
            'AUTO_INDENT': True,             # Use Auto Indent
            'AUTO_TRIM_WS' : False,          # Trim whitespace on save
            'AUTO_RELOAD' : False,           # Automatically reload files?
@@ -227,6 +229,10 @@ class Profile(dict):
 
         """
         try:
+            # Only write if given an absolute path
+            if not os.path.isabs(path):
+                return False
+
             self.Set('MYPROFILE', path)
             UpdateProfileLoader()
             fhandle = open(path, 'wb')
