@@ -12,14 +12,13 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 59373 2009-03-06 17:20:59Z CJP $"
-__revision__ = "$Revision: 59373 $"
+__svnid__ = "$Id: util.py 60250 2009-04-20 03:35:30Z CJP $"
+__revision__ = "$Revision: 60250 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import os
 import sys
-import stat
 import types
 import mimetypes
 import encodings
@@ -36,6 +35,7 @@ import syntax.syntax as syntax
 import syntax.synglob as synglob
 import eclib # TODO: Temporary till refs to HexToRGB and AdjustColor are
              #       no longer used anywhere
+import ebmlib
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -297,23 +297,12 @@ def FilterFiles(file_list):
 
     """
     good = list()
-    checker = FileTypeChecker()
+    checker = ebmlib.FileTypeChecker()
     for path in file_list:
         if not checker.IsBinary(path):
             good.append(path)
     return good
 
-def GetFileModTime(file_name):
-    """Returns the time that the given file was last modified on
-    @param file_name: path of file to get mtime of
-
-    """
-    try:
-        mod_time = os.path.getmtime(file_name)
-    except (OSError, EnvironmentError):
-        mod_time = 0
-    return mod_time
-
 def GetFileType(fname):
     """Get what the type of the file is as Editra sees it
     in a formatted string.
@@ -384,31 +373,6 @@ def GetFileWriter(file_name, enc='utf-8'):
         writer = file_h
     return writer
 
-def GetUniqueName(path, name):
-    """Make a file name that will be unique in case a file of the
-    same name already exists at that path.
-    @param path: Root path to folder of files destination
-    @param name: desired file name base
-    @return: string
-
-    """
-    tmpname = os.path.join(path, name)
-    if os.path.exists(tmpname):
-        if '.' not in name:
-            ext = ''
-            fbase = name
-        else:
-            ext = '.' + name.split('.')[-1]
-            fbase = name[:-1 * len(ext)]
-
-        inc = len([x for x in os.listdir(path) if x.startswith(fbase)])
-        tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
-        while os.path.exists(tmpname):
-            inc = inc + 1
-            tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
-
-    return tmpname
-
 def GetPathName(path):
     """Gets the path minus filename
     @param path: full path to get base of
@@ -444,17 +408,6 @@ def GetFileName(path):
     """
     return os.path.split(path)[-1]
 
-def GetFileSize(path):
-    """Get the size of the file at a given path
-    @param path: Path to file
-    @return: long
-
-    """
-    try:
-        return os.stat(path)[stat.ST_SIZE]
-    except:
-        return 0
-
 def GetExtension(file_str):
     """Gets last atom at end of string as extension if
     no extension whole string is returned
@@ -463,45 +416,6 @@ def GetExtension(file_str):
     """
     return file_str.split('.')[-1]
 
-def MakeNewFile(path, name):
-    """Make a new file at the given path with the given name.
-    If the file already exists, the given name will be changed to
-    a unique name in the form of name + -NUMBER + .extension
-    @param path: path to directory to create file in
-    @param name: desired name of file
-    @return: Tuple of (success?, Path of new file OR Error message)
-
-    """
-    if not os.path.isdir(path):
-        path = os.path.dirname(path)
-    fname = GetUniqueName(path, name)
-
-    try:
-        open(fname, 'w').close()
-    except (IOError, OSError), msg:
-        return (False, str(msg))
-
-    return (True, fname)
-
-def MakeNewFolder(path, name):
-    """Make a new folder at the given path with the given name.
-    If the folder already exists, the given name will be changed to
-    a unique name in the form of name + -NUMBER.
-    @param path: path to create folder on
-    @param name: desired name for folder
-    @return: Tuple of (success?, new dirname OR Error message)
-
-    """
-    if not os.path.isdir(path):
-        path = os.path.dirname(path)
-    folder = GetUniqueName(path, name)
-    try:
-        os.mkdir(folder)
-    except (OSError, IOError), msg:
-        return (False, str(msg))
-
-    return (True, folder)
-
 def HasConfigDir(loc=u""):
     """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
@@ -760,29 +674,6 @@ def GetProxyOpener(proxy_set):
     return opener
 
 #---- GUI helper functions ----#
-def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
-    """ Brighten/Darken input colour by percent and adjust alpha
-    channel if needed. Returns the modified color.
-    @param color: color object to adjust
-    @type color: wx.Color
-    @param percent: percent to adjust +(brighten) or -(darken)
-    @type percent: int
-    @keyword alpha: Value to adjust alpha channel to
-    @note: DON'T USE THIS FUNCTION use the one in eclib instead
-           this will be removed at some point.
-
-    """
-    return eclib.AdjustColour(color, percent, alpha)
-
-def HexToRGB(hex_str):
-    """Returns a list of red/green/blue values from a
-    hex string.
-    @param hex_str: hex string to convert to rgb
-    @note: DON'T USE THIS FUNCTION use the one in eclib instead
-           this will be removed at some point.
-
-    """
-    return eclib.HexToRGB(hex_str)
 
 def SetWindowIcon(window):
     """Sets the given windows icon to be the programs
@@ -802,59 +693,6 @@ def SetWindowIcon(window):
 
 #-----------------------------------------------------------------------------#
 
-class FileTypeChecker:
-    """File type checker and recognizer"""
-    TXTCHARS = ''.join(map(chr, [7,8,9,10,12,13,27] + range(0x20, 0x100)))
-    ALLBYTES = ''.join(map(chr, range(256)))
-
-    def __init__(self, preread=4096):
-        """Create the FileTypeChecker
-        @keyword preread: number of bytes to read for checking file type
-
-        """
-        # Attributes
-        self._preread = preread
-
-    def _GetHandle(self, fname):
-        """Get a file handle for reading
-        @param fname: filename
-        @return: file object or None
-
-        """
-        try:
-            handle = open(fname, 'rb')
-        except:
-            handle = None
-        return handle
-
-    def IsBinary(self, fname):
-        """Is the file made up of binary data
-        @param fname: filename to check
-        @return: bool
-
-        """
-        handle = self._GetHandle(fname)
-        if handle is not None:
-            bytes = handle.read(self._preread)
-            handle.close()
-            nontext = bytes.translate(FileTypeChecker.ALLBYTES,
-                                      FileTypeChecker.TXTCHARS)
-            return bool(nontext)
-        else:
-            return False
-
-    def IsReadableText(self, fname):
-        """Is the given path readable as text
-        @param fname: filename
-
-        """
-        f_ok = False
-        if os.access(fname, os.R_OK):
-            f_ok = not self.IsBinary(fname)
-        return f_ok
-
-#-----------------------------------------------------------------------------#
-
 class IntValidator(wx.PyValidator):
     """A Generic integer validator"""
     def __init__(self, min_=0, max_=0):
