@@ -9,8 +9,8 @@
 """Unittests for EdFile"""
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: testEdFile.py 59010 2009-02-19 03:53:56Z CJP $"
-__revision__ = "$Revision: 59010 $"
+__svnid__ = "$Id: testEdFile.py 60249 2009-04-20 01:18:29Z CJP $"
+__revision__ = "$Revision: 60249 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -25,6 +25,7 @@ import common
 # Module(s) to test
 import ed_txt
 import util
+import ebmlib
 
 #-----------------------------------------------------------------------------#
 
@@ -33,7 +34,7 @@ class EdFileTest(unittest.TestCase):
         self.app = common.EdApp(False)
         self.path = os.path.abspath('./data/test_read_utf8.txt')
         self.file = ed_txt.EdFile(self.path)
-        self.mtime = util.GetFileModTime(self.path)
+        self.mtime = ebmlib.GetFileModTime(self.path)
 
     def tearDown(self):
         self.file.Close()
